package com.sun.xml.internal.ws.api.pipe;

import com.sun.xml.internal.ws.api.pipe.helper.PipeAdapter;
import com.sun.istack.internal.NotNull;
import java.util.Iterator;
import com.sun.xml.internal.ws.assembler.MetroTubelineAssembler;
import java.util.logging.Level;
import com.sun.xml.internal.ws.util.ServiceFinder;
import com.sun.istack.internal.Nullable;
import com.sun.xml.internal.ws.api.server.Container;
import com.sun.xml.internal.ws.api.BindingID;
import java.util.logging.Logger;

public abstract class TubelineAssemblerFactory
{
    private static final Logger logger;
    
    public TubelineAssemblerFactory() {
    }
    
    public abstract TubelineAssembler doCreate(BindingID p0);
    
    @Deprecated
    public static TubelineAssembler create(ClassLoader classLoader, BindingID bindingId) {
        return create(classLoader, bindingId, null);
    }
    
    public static TubelineAssembler create(ClassLoader classLoader, BindingID bindingId, @Nullable Container container) {
        if (container != null) {
            TubelineAssemblerFactory taf = (TubelineAssemblerFactory)container.getSPI(TubelineAssemblerFactory.class);
            if (taf != null) {
                TubelineAssembler a = taf.doCreate(bindingId);
                if (a != null) {
                    return a;
                }
            }
        }
        for (TubelineAssemblerFactory factory : ServiceFinder.find(TubelineAssemblerFactory.class, classLoader)) {
            TubelineAssembler assembler = factory.doCreate(bindingId);
            if (assembler != null) {
                TubelineAssemblerFactory.logger.log(Level.FINE, "{0} successfully created {1}", new Object[] { factory.getClass(), assembler });
                return assembler;
            }
            continue;
        }
        for (PipelineAssemblerFactory factory2 : ServiceFinder.find(PipelineAssemblerFactory.class, classLoader)) {
            PipelineAssembler assembler2 = factory2.doCreate(bindingId);
            if (assembler2 != null) {
                TubelineAssemblerFactory.logger.log(Level.FINE, "{0} successfully created {1}", new Object[] { factory2.getClass(), assembler2 });
                return new TubelineAssemblerAdapter(assembler2);
            }
            continue;
        }
        return new MetroTubelineAssembler(bindingId, MetroTubelineAssembler.JAXWS_TUBES_CONFIG_NAMES);
    }
    
    static {
        logger = Logger.getLogger(TubelineAssemblerFactory.class.getName());
    }
    
    private static class TubelineAssemblerAdapter implements TubelineAssembler
    {
        private PipelineAssembler assembler;
        
        TubelineAssemblerAdapter(PipelineAssembler assembler) {
            this.assembler = assembler;
        }
        
        @NotNull
        @Override
        public Tube createClient(@NotNull ClientTubeAssemblerContext context) {
            ClientPipeAssemblerContext ctxt = new ClientPipeAssemblerContext(context.getAddress(), context.getWsdlModel(), context.getService(), context.getBinding(), context.getContainer());
            return PipeAdapter.adapt(this.assembler.createClient(ctxt));
        }
        
        @NotNull
        @Override
        public Tube createServer(@NotNull ServerTubeAssemblerContext context) {
            if (!(context instanceof ServerPipeAssemblerContext)) {
                throw new IllegalArgumentException("{0} is not instance of ServerPipeAssemblerContext");
            }
            return PipeAdapter.adapt(this.assembler.createServer((ServerPipeAssemblerContext)context));
        }
    }
}
